@@ -1,6 +1,6 @@
 #    test_repack_tarball.py -- Testsuite for repacking of tarballs
 #    Copyright (C) 2007 James Westby <jw+debian@jameswestby.net>
-#    
+#
 #    This file is part of bzr-builddeb.
 #
 #    bzr-builddeb is free software; you can redistribute it and/or modify
@@ -24,13 +24,13 @@ import tarfile
 from bzrlib.errors import (NoSuchFile,
                            FileExists,
                            )
-from bzrlib.tests import TestCaseInTempDir
-
 from bzrlib.plugins.builddeb.errors import UnsupportedRepackFormat
 from bzrlib.plugins.builddeb.repack_tarball import (
         repack_tarball,
+        get_filetype,
         get_repacker_class,
         )
+from bzrlib.plugins.builddeb.tests import TestCaseInTempDir
 
 
 def touch(filename):
@@ -66,11 +66,13 @@ class TestRepackTarball(TestCaseInTempDir):
 
     def test_create_old_tarball(self):
         self.create_old_tarball()
-        self.failUnlessExists(self.old_tarball)
+        self.assertPathExists(self.old_tarball)
 
     def test_repack_tarball_non_extant(self):
         error = NoSuchFile
-        if (get_repacker_class(self.old_tarball) is None):
+        old_format = get_filetype(self.old_tarball)
+        new_format = get_filetype(self.new_tarball)
+        if (get_repacker_class(old_format, new_format) is None):
             # directory, can't really be detected remotely, so we have a
             # error that could mean two things
             error = UnsupportedRepackFormat
@@ -86,15 +88,17 @@ class TestRepackTarball(TestCaseInTempDir):
     def test_repack_tarball_creates_new(self):
         self.create_old_tarball()
         repack_tarball(self.old_tarball, self.new_tarball)
-        self.failUnlessExists(self.old_tarball)
-        self.failUnlessExists(self.new_tarball)
+        self.assertPathExists(self.old_tarball)
+        self.assertPathExists(self.new_tarball)
 
     def test_repack_tarball_contents(self):
         self.create_old_tarball()
         repack_tarball(self.old_tarball, self.new_tarball)
         tar = tarfile.open(self.new_tarball, 'r:gz')
         try:
-            members = tar.getnames()
+            # The tarfile module returns member names of directories with a
+            # trailing slash, in Python 2.5 *only*.
+            members = [x.rstrip("/") for x in tar.getnames()]
         finally:
             tar.close()
         self.assertEqual(members,
@@ -105,20 +109,20 @@ class TestRepackTarball(TestCaseInTempDir):
         self.create_old_tarball()
         target_dir = 'tarballs'
         repack_tarball(self.old_tarball, self.new_tarball, target_dir=target_dir)
-        self.failUnlessExists(target_dir)
-        self.failUnlessExists(os.path.join(target_dir, self.new_tarball))
-        self.failUnlessExists(self.old_tarball)
-        self.failIfExists(self.new_tarball)
+        self.assertPathExists(target_dir)
+        self.assertPathExists(os.path.join(target_dir, self.new_tarball))
+        self.assertPathExists(self.old_tarball)
+        self.assertPathDoesNotExist(self.new_tarball)
 
     def test_repack_tarball_with_target_dir_exists(self):
         self.create_old_tarball()
         target_dir = 'tarballs'
         os.mkdir(target_dir)
         repack_tarball(self.old_tarball, self.new_tarball, target_dir=target_dir)
-        self.failUnlessExists(target_dir)
-        self.failUnlessExists(os.path.join(target_dir, self.new_tarball))
-        self.failUnlessExists(self.old_tarball)
-        self.failIfExists(self.new_tarball)
+        self.assertPathExists(target_dir)
+        self.assertPathExists(os.path.join(target_dir, self.new_tarball))
+        self.assertPathExists(self.old_tarball)
+        self.assertPathDoesNotExist(self.new_tarball)
 
     def test_repack_tarball_with_target_dir_not_dir(self):
         self.create_old_tarball()
@@ -127,7 +131,10 @@ class TestRepackTarball(TestCaseInTempDir):
         # transport gives NoSuchFile rather than NotADirectory for this
         self.assertRaises(NoSuchFile, repack_tarball, self.old_tarball,
                           self.new_tarball, target_dir=target_dir)
-        self.failUnlessExists(self.old_tarball)
-        self.failIfExists(self.new_tarball)
-        self.failIfExists(os.path.join(target_dir, self.new_tarball))
-        self.failUnlessExists(target_dir)
+        self.assertPathExists(self.old_tarball)
+        self.assertPathDoesNotExist(self.new_tarball)
+        self.assertPathDoesNotExist(os.path.join(target_dir, self.new_tarball))
+        self.assertPathExists(target_dir)
+
+
+
