@@ -18,12 +18,15 @@
 #    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
 #
 
+import bz2
+import gzip
 try:
     import hashlib as md5
 except ImportError:
     import md5
 import os
 import shutil
+import tarfile
 
 try:
     from debian.changelog import Changelog, Version
@@ -31,41 +34,54 @@ except ImportError:
     # Prior to 0.1.15 the debian module was called debian_bundle
     from debian_bundle.changelog import Changelog, Version
 
-from bzrlib.plugins.builddeb.config import DebBuildConfig
-from bzrlib.plugins.builddeb.errors import (MissingChangelogError,
-                AddChangelogError,
-                NoPreviousUpload,
-                UnknownDistribution,
-                )
-from bzrlib.plugins.builddeb.tests import SourcePackageBuilder
+from bzrlib.plugins.builddeb.config import (
+    BUILD_TYPE_MERGE,
+    BUILD_TYPE_NATIVE,
+    BUILD_TYPE_NORMAL,
+    )
+from bzrlib.plugins.builddeb.errors import (
+    MissingChangelogError,
+    AddChangelogError,
+    InconsistentSourceFormatError,
+    NoPreviousUpload,
+    )
+from bzrlib.plugins.builddeb.tests import (
+    LzmaFeature,
+    SourcePackageBuilder,
+    TestCaseInTempDir,
+    TestCaseWithTransport,
+    )
 from bzrlib.plugins.builddeb.util import (
-                  dget,
-                  dget_changes,
-                  find_bugs_fixed,
-                  find_changelog,
-                  find_extra_authors,
-                  find_last_distribution,
-                  _find_previous_upload,
-                  find_thanks,
-                  get_export_upstream_revision,
-                  get_commit_info_from_changelog,
-                  get_snapshot_revision,
-                  lookup_distribution,
-                  move_file_if_different,
-                  get_parent_dir,
-                  recursive_copy,
-                  safe_decode,
-                  strip_changelog_message,
-                  suite_to_distribution,
-                  tarball_name,
-                  write_if_different,
-                  )
+    component_from_orig_tarball,
+    dget,
+    dget_changes,
+    extract_orig_tarballs,
+    find_bugs_fixed,
+    find_changelog,
+    find_extra_authors,
+    find_last_distribution,
+    _find_previous_upload,
+    find_thanks,
+    get_commit_info_from_changelog,
+    get_source_format,
+    guess_build_type,
+    lookup_distribution,
+    move_file_if_different,
+    get_parent_dir,
+    recursive_copy,
+    safe_decode,
+    strip_changelog_message,
+    suite_to_distribution,
+    tarball_name,
+    tree_contains_upstream_source,
+    write_if_different,
+    )
 
 from bzrlib import errors as bzr_errors
-from bzrlib.tests import (TestCaseWithTransport,
-                          TestCaseInTempDir,
-                          TestCase,
-                          )
+from bzrlib.tests import (
+    SymlinkFeature,
+    TestCase,
+    )
 
 
 class RecursiveCopyTests(TestCaseInTempDir):
@@ -83,15 +99,15 @@ class RecursiveCopyTests(TestCaseInTempDir):
             f.close()
         os.mkdir('b/g')
         recursive_copy('a', 'b')
-        self.failUnlessExists('a')
-        self.failUnlessExists('b')
-        self.failUnlessExists('c')
-        self.failUnlessExists('b/d')
-        self.failUnlessExists('b/d/e')
-        self.failUnlessExists('b/f')
-        self.failUnlessExists('a/d')
-        self.failUnlessExists('a/d/e')
-        self.failUnlessExists('a/f')
+        self.assertPathExists('a')
+        self.assertPathExists('b')
+        self.assertPathExists('c')
+        self.assertPathExists('b/d')
+        self.assertPathExists('b/d/e')
+        self.assertPathExists('b/f')
+        self.assertPathExists('a/d')
+        self.assertPathExists('a/d/e')
+        self.assertPathExists('a/f')
 
 
 class SafeDecodeTests(TestCase):
@@ -162,6 +178,16 @@ bzr-builddeb (0.16.2) unstable; urgency=low
         self.assertEqual(str(cl), cl_block1)
         self.assertEqual(lq, True)
 
+    def test_find_changelog_lq_unversioned_debian_symlink(self):
+        # LarstiQ mode, but with an unversioned "debian" -> "." symlink.
+        # Bug 619295
+        self.requireFeature(SymlinkFeature)
+        tree = self.make_branch_and_tree('.')
+        self.write_changelog('changelog')
+        tree.add(['changelog'])
+        os.symlink('.', 'debian')
+        self.assertRaises(AddChangelogError, find_changelog, tree, True)
+
     def test_find_changelog_nomerge_lq(self):
         tree = self.make_branch_and_tree('.')
         self.write_changelog('changelog')
@@ -179,6 +205,7 @@ bzr-builddeb (0.16.2) unstable; urgency=low
 
     def test_find_changelog_symlink(self):
         """When there was a symlink debian -> . then the code used to break"""
+        self.requireFeature(SymlinkFeature)
         tree = self.make_branch_and_tree('.')
         self.write_changelog('changelog')
         tree.add(['changelog'])
@@ -247,35 +274,16 @@ class StripChangelogMessageTests(TestCase):
 class TarballNameTests(TestCase):
 
     def test_tarball_name(self):
-        self.assertEqual(tarball_name("package", "0.1"),
+        self.assertEqual(tarball_name("package", "0.1", None),
                 "package_0.1.orig.tar.gz")
-        self.assertEqual(tarball_name("package", Version("0.1")),
+        self.assertEqual(tarball_name("package", Version("0.1"), None),
                 "package_0.1.orig.tar.gz")
-        self.assertEqual(tarball_name("package", Version("0.1"),
+        self.assertEqual(tarball_name("package", Version("0.1"), None,
                     format='bz2'), "package_0.1.orig.tar.bz2")
-        self.assertEqual(tarball_name("package", Version("0.1"),
+        self.assertEqual(tarball_name("package", Version("0.1"), None,
                     format='lzma'), "package_0.1.orig.tar.lzma")
-
-
-class GetRevisionSnapshotTests(TestCase):
-
-    def test_with_snapshot(self):
-        self.assertEquals("30", get_snapshot_revision("0.4.4~bzr30"))
-
-    def test_with_snapshot_plus(self):
-        self.assertEquals("30", get_snapshot_revision("0.4.4+bzr30"))
-
-    def test_without_snapshot(self):
-        self.assertEquals(None, get_snapshot_revision("0.4.4"))
-
-    def test_non_numeric_snapshot(self):
-        self.assertEquals(None, get_snapshot_revision("0.4.4~bzra"))
-
-    def test_with_svn_snapshot(self):
-        self.assertEquals("svn:4242", get_snapshot_revision("0.4.4~svn4242"))
-
-    def test_with_svn_snapshot_plus(self):
-        self.assertEquals("svn:2424", get_snapshot_revision("0.4.4+svn2424"))
+        self.assertEqual(tarball_name("package", Version("0.1"), "la",
+                    format='lzma'), "package_0.1.orig-la.tar.lzma")
 
 
 class SuiteToDistributionTests(TestCase):
@@ -328,13 +336,13 @@ class MoveFileTests(TestCaseInTempDir):
     def test_move_file_non_extant(self):
         self.build_tree(['a'])
         move_file_if_different('a', 'b', None)
-        self.failIfExists('a')
-        self.failUnlessExists('b')
+        self.assertPathDoesNotExist('a')
+        self.assertPathExists('b')
 
     def test_move_file_samefile(self):
         self.build_tree(['a'])
         move_file_if_different('a', 'a', None)
-        self.failUnlessExists('a')
+        self.assertPathExists('a')
 
     def test_move_file_same_md5(self):
         self.build_tree(['a'])
@@ -346,8 +354,8 @@ class MoveFileTests(TestCaseInTempDir):
             f.close()
         shutil.copy('a', 'b')
         move_file_if_different('a', 'b', md5sum.hexdigest())
-        self.failUnlessExists('a')
-        self.failUnlessExists('b')
+        self.assertPathExists('a')
+        self.assertPathExists('b')
 
     def test_move_file_diff_md5(self):
         self.build_tree(['a', 'b'])
@@ -367,8 +375,8 @@ class MoveFileTests(TestCaseInTempDir):
         b_hexdigest = md5sum.hexdigest()
         self.assertNotEqual(a_hexdigest, b_hexdigest)
         move_file_if_different('a', 'b', a_hexdigest)
-        self.failIfExists('a')
-        self.failUnlessExists('b')
+        self.assertPathDoesNotExist('a')
+        self.assertPathExists('b')
         md5sum = md5.md5()
         f = open('b', 'rb')
         try:
@@ -382,23 +390,23 @@ class WriteFileTests(TestCaseInTempDir):
 
     def test_write_non_extant(self):
         write_if_different("foo", 'a')
-        self.failUnlessExists('a')
+        self.assertPathExists('a')
         self.check_file_contents('a', "foo")
 
     def test_write_file_same(self):
         write_if_different("foo", 'a')
-        self.failUnlessExists('a')
+        self.assertPathExists('a')
         self.check_file_contents('a', "foo")
         write_if_different("foo", 'a')
-        self.failUnlessExists('a')
+        self.assertPathExists('a')
         self.check_file_contents('a', "foo")
 
     def test_write_file_different(self):
         write_if_different("foo", 'a')
-        self.failUnlessExists('a')
+        self.assertPathExists('a')
         self.check_file_contents('a', "foo")
         write_if_different("bar", 'a')
-        self.failUnlessExists('a')
+        self.assertPathExists('a')
         self.check_file_contents('a', "bar")
 
 
@@ -411,9 +419,9 @@ class DgetTests(TestCaseWithTransport):
         builder.build()
         self.build_tree(["target/"])
         dget(builder.dsc_name(), 'target')
-        self.failUnlessExists(os.path.join("target", builder.dsc_name()))
-        self.failUnlessExists(os.path.join("target", builder.tar_name()))
-        self.failUnlessExists(os.path.join("target", builder.diff_name()))
+        self.assertPathExists(os.path.join("target", builder.dsc_name()))
+        self.assertPathExists(os.path.join("target", builder.tar_name()))
+        self.assertPathExists(os.path.join("target", builder.diff_name()))
 
     def test_dget_transport(self):
         builder = SourcePackageBuilder("package", Version("0.1-1"))
@@ -422,9 +430,9 @@ class DgetTests(TestCaseWithTransport):
         builder.build()
         self.build_tree(["target/"])
         dget(self.get_url(builder.dsc_name()), 'target')
-        self.failUnlessExists(os.path.join("target", builder.dsc_name()))
-        self.failUnlessExists(os.path.join("target", builder.tar_name()))
-        self.failUnlessExists(os.path.join("target", builder.diff_name()))
+        self.assertPathExists(os.path.join("target", builder.dsc_name()))
+        self.assertPathExists(os.path.join("target", builder.tar_name()))
+        self.assertPathExists(os.path.join("target", builder.diff_name()))
 
     def test_dget_missing_dsc(self):
         builder = SourcePackageBuilder("package", Version("0.1-1"))
@@ -460,10 +468,10 @@ class DgetTests(TestCaseWithTransport):
         builder.build()
         self.build_tree(["target/"])
         dget_changes(builder.changes_name(), 'target')
-        self.failUnlessExists(os.path.join("target", builder.dsc_name()))
-        self.failUnlessExists(os.path.join("target", builder.tar_name()))
-        self.failUnlessExists(os.path.join("target", builder.diff_name()))
-        self.failUnlessExists(os.path.join("target", builder.changes_name()))
+        self.assertPathExists(os.path.join("target", builder.dsc_name()))
+        self.assertPathExists(os.path.join("target", builder.tar_name()))
+        self.assertPathExists(os.path.join("target", builder.diff_name()))
+        self.assertPathExists(os.path.join("target", builder.changes_name()))
 
 
 class ParentDirTests(TestCase):
@@ -708,32 +716,6 @@ class FindLastDistributionTests(TestCase):
         self.assertEquals(None, find_last_distribution(changelog))
 
 
-class GetExportUpstreamRevisionTests(TestCase):
-
-    def test_snapshot_rev(self):
-        config = DebBuildConfig([])
-        self.assertEquals("34", 
-            get_export_upstream_revision(config, Version("0.1+bzr34-1")))
-
-    def test_export_upstream_rev(self):
-        config = DebBuildConfig([
-            ({"BUILDDEB": {"export-upstream-revision": "tag:foobar"}}, True)])
-        self.assertEquals("tag:foobar", 
-            get_export_upstream_revision(config, Version("0.1-1")))
-
-    def test_export_upstream_rev_var(self):
-        config = DebBuildConfig([({"BUILDDEB": 
-            {"export-upstream-revision": "tag:foobar-$UPSTREAM_VERSION"}},
-            True)])
-        self.assertEquals("tag:foobar-0.1", 
-            get_export_upstream_revision(config, Version("0.1-1")))
-
-    def test_export_upstream_rev_not_set(self):
-        config = DebBuildConfig([])
-        self.assertEquals(None, 
-            get_export_upstream_revision(config, Version("0.1-1")))
-
-
 class FindPreviousUploadTests(TestCase):
 
     def make_changelog(self, versions_and_distributions):
@@ -761,10 +743,15 @@ class FindPreviousUploadTests(TestCase):
                 ("0.1-1.1", "unstable"), ("0.1-2", "maverick")])
         self.assertEqual(Version("0.1-1"), _find_previous_upload(cl))
 
+    def test_find_previous_upload_ubuntu_pocket(self):
+        cl = self.make_changelog([("0.1-1", "lucid-updates"),
+                ("0.1-2", "lucid-updates")])
+        self.assertEqual(Version("0.1-1"), _find_previous_upload(cl))
+
     def test_find_previous_upload_unknown(self):
         cl = self.make_changelog([("0.1-1", "lucid"),
-                ("0.1-2", "UNRELEASED")])
-        self.assertRaises(UnknownDistribution, _find_previous_upload, cl)
+                ("0.1-2", "dunno")])
+        self.assertRaises(NoPreviousUpload, _find_previous_upload, cl)
 
     def test_find_previous_upload_missing(self):
         cl = self.make_changelog([("0.1-1", "unstable"),
@@ -772,3 +759,236 @@ class FindPreviousUploadTests(TestCase):
         self.assertRaises(NoPreviousUpload, _find_previous_upload, cl)
         cl = self.make_changelog([("0.1-1", "unstable")])
         self.assertRaises(NoPreviousUpload, _find_previous_upload, cl)
+
+    def test_find_previous_upload_unreleased(self):
+        cl = self.make_changelog([("0.1-1", "unstable"),
+                ("0.1-2", "UNRELEASED")])
+        self.assertEqual(Version("0.1-1"), _find_previous_upload(cl))
+
+
+class SourceFormatTests(TestCaseWithTransport):
+
+    def test_no_source_format_file(self):
+        tree = self.make_branch_and_tree('.')
+        self.assertEquals("1.0", get_source_format(tree))
+
+    def test_source_format_newline(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree_contents([("debian/", ), ("debian/source/",),
+            ("debian/source/format", "3.0 (native)\n")])
+        tree.add(["debian", "debian/source", "debian/source/format"])
+        self.assertEquals("3.0 (native)", get_source_format(tree))
+
+    def test_source_format(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree_contents([("debian/",), ("debian/source/",),
+            ("debian/source/format", "3.0 (quilt)")])
+        tree.add(["debian", "debian/source", "debian/source/format"])
+        self.assertEquals("3.0 (quilt)", get_source_format(tree))
+
+    def test_source_format_file_unversioned(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree_contents([("debian/",), ("debian/source/",),
+            ("debian/source/format", "3.0 (quilt)")])
+        self.assertEquals("3.0 (quilt)", get_source_format(tree))
+
+
+class GuessBuildTypeTests(TestCaseWithTransport):
+    """Tests for guess_build_type."""
+
+    def writeVersionFile(self, tree, format_string):
+        """Write a Debian source format file.
+
+        :param tree: Tree to write to.
+        :param format_string: Format string to write.
+        """
+        self.build_tree_contents([("debian/",), ("debian/source/",),
+            ("debian/source/format", format_string)])
+        tree.add(["debian", "debian/source", "debian/source/format"])
+
+    def test_normal_source_format(self):
+        # Normal source format -> NORMAL
+        tree = self.make_branch_and_tree('.')
+        self.writeVersionFile(tree, "3.0 (quilt)")
+        self.assertEquals(BUILD_TYPE_NORMAL, guess_build_type(tree, None, True))
+
+    def test_normal_source_format_merge(self):
+        # Normal source format without upstream source -> MERGE
+        tree = self.make_branch_and_tree('.')
+        self.writeVersionFile(tree, "3.0 (quilt)")
+        self.assertEquals(BUILD_TYPE_MERGE, guess_build_type(tree, None, False))
+
+    def test_native_source_format(self):
+        # Native source format -> NATIVE
+        tree = self.make_branch_and_tree('.')
+        self.writeVersionFile(tree, "3.0 (native)")
+        self.assertEquals(BUILD_TYPE_NATIVE, guess_build_type(tree, None, True))
+
+    def test_prev_version_native(self):
+        # Native package version -> NATIVE
+        tree = self.make_branch_and_tree('.')
+        self.assertEquals(BUILD_TYPE_NATIVE,
+            guess_build_type(tree, Version("1.0"), True))
+
+    def test_empty(self):
+        # Empty tree and a non-native package -> NORMAL
+        tree = self.make_branch_and_tree('.')
+        self.assertEquals(BUILD_TYPE_NORMAL,
+            guess_build_type(tree, Version("1.0-1"), None))
+
+    def test_no_upstream_source(self):
+        # No upstream source code and a non-native package -> MERGE
+        tree = self.make_branch_and_tree('.')
+        tree.mkdir("debian")
+        self.assertEquals(BUILD_TYPE_MERGE,
+            guess_build_type(tree, Version("1.0-1"), False))
+
+    def test_default(self):
+        # Upstream source code and a non-native package -> NORMAL
+        tree = self.make_branch_and_tree('.')
+        self.assertEquals(BUILD_TYPE_NORMAL,
+            guess_build_type(tree, Version("1.0-1"), True))
+
+    def test_inconsistent(self):
+        # If version string and source format disagree on whether the package is native,
+        # raise an exception.
+        tree = self.make_branch_and_tree('.')
+        self.writeVersionFile(tree, "3.0 (native)")
+        e = self.assertRaises(InconsistentSourceFormatError, guess_build_type, tree,
+            Version("1.0-1"), True)
+        self.assertEquals(
+            "Inconsistency between source format and version: version is not native, "
+            "format is native.", str(e))
+
+
+class TestExtractOrigTarballs(TestCaseInTempDir):
+
+    def create_tarball(self, package, version, compression, part=None):
+        basedir = "%s-%s" % (package, version)
+        os.mkdir(basedir)
+        try:
+            f = open(os.path.join(basedir, "README"), 'w')
+            try:
+                f.write("Hi\n")
+            finally:
+                f.close()
+            prefix = "%s_%s.orig" % (package, version)
+            if part is not None:
+                prefix += "-%s" % part
+            tar_path = os.path.abspath(prefix + ".tar." + compression)
+            if compression == "gz":
+                f = gzip.GzipFile(tar_path, "w")
+            elif compression == "bz2":
+                f = bz2.BZ2File(tar_path, "w")
+            elif compression == "lzma":
+                import lzma
+                f = lzma.LZMAFile(tar_path, "w")
+            else:
+                raise AssertionError("Unknown compressin type %r" %
+                    compression)
+            try:
+                tf = tarfile.open(None, 'w', f)
+                try:
+                    tf.add(basedir)
+                finally:
+                    tf.close()
+            finally:
+                f.close()
+        finally:
+            shutil.rmtree(basedir)
+        return tar_path
+
+    def test_single_orig_tar_gz(self):
+        tar_path = self.create_tarball("package", "0.1", "gz")
+        os.mkdir("target")
+        extract_orig_tarballs([(tar_path, None)], "target",
+            strip_components=1)
+        self.assertEquals(os.listdir("target"), ["README"])
+
+    def test_single_orig_tar_bz2(self):
+        tar_path = self.create_tarball("package", "0.1", "bz2")
+        os.mkdir("target")
+        extract_orig_tarballs([(tar_path, None)], "target",
+            strip_components=1)
+        self.assertEquals(os.listdir("target"), ["README"])
+
+    def test_single_orig_tar_lzma(self):
+        self.requireFeature(LzmaFeature)
+        tar_path = self.create_tarball("package", "0.1", "lzma")
+        os.mkdir("target")
+        extract_orig_tarballs([(tar_path, None)], "target",
+            strip_components=1)
+        self.assertEquals(os.listdir("target"), ["README"])
+
+    def test_multiple_tarballs(self):
+        base_tar_path = self.create_tarball("package", "0.1", "bz2")
+        tar_path_extra = self.create_tarball("package", "0.1", "bz2", part="extra")
+        os.mkdir("target")
+        extract_orig_tarballs([(base_tar_path, None), (tar_path_extra, "extra")], "target",
+            strip_components=1)
+        self.assertEquals(sorted(os.listdir("target")),
+            sorted(["README", "extra"]))
+
+
+class ComponentFromOrigTarballTests(TestCase):
+
+    def test_base_tarball(self):
+        self.assertIs(None,
+            component_from_orig_tarball("foo_0.1.orig.tar.gz", "foo", "0.1"))
+        self.assertRaises(ValueError,
+            component_from_orig_tarball, "foo_0.1.orig.tar.gz", "bar", "0.1")
+
+    def test_invalid_extension(self):
+        self.assertRaises(ValueError,
+            component_from_orig_tarball, "foo_0.1.orig.unknown", "foo", "0.1")
+
+    def test_component(self):
+        self.assertEquals("comp",
+            component_from_orig_tarball("foo_0.1.orig-comp.tar.gz", "foo", "0.1"))
+        self.assertEquals("comp-dash",
+            component_from_orig_tarball("foo_0.1.orig-comp-dash.tar.gz", "foo", "0.1"))
+
+    def test_invalid_character(self):
+        self.assertRaises(ValueError,
+            component_from_orig_tarball, "foo_0.1.orig;.tar.gz", "foo", "0.1")
+
+
+class TreeContainsUpstreamSourceTests(TestCaseWithTransport):
+
+    def test_empty(self):
+        tree = self.make_branch_and_tree('.')
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
+        self.assertIs(None, tree_contains_upstream_source(tree))
+
+    def test_debian_dir_only(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['debian/'])
+        tree.add(['debian'])
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
+        self.assertFalse(tree_contains_upstream_source(tree))
+
+    def test_debian_dir_and_bzr_builddeb(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['debian/', '.bzr-builddeb/'])
+        tree.add(['debian', '.bzr-builddeb'])
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
+        self.assertFalse(tree_contains_upstream_source(tree))
+
+    def test_with_upstream_source(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['debian/', 'src/'])
+        tree.add(['debian', 'src'])
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
+        self.assertTrue(tree_contains_upstream_source(tree))
+
+    def test_with_unversioned_extra_data(self):
+        tree = self.make_branch_and_tree('.')
+        self.build_tree(['debian/', 'x'])
+        tree.add(['debian'])
+        tree.lock_read()
+        self.addCleanup(tree.unlock)
+        self.assertFalse(tree_contains_upstream_source(tree))
